from __future__ import absolute_import

from django.core.exceptions import ValidationError as DjangoValidationError
from rest_framework import serializers

from netfields import fields


class NetfieldsField(serializers.CharField):
    def __init__(self, *args, **kwargs):
        super(NetfieldsField, self).__init__(*args, **kwargs)
        self.validators.append(self._validate_netaddr)

    def _validate_netaddr(self, value):
        """Convert Django validation errors to DRF validation errors.
        """
        try:
            self.netfields_type(value).to_python(value)
        except DjangoValidationError as e:
            raise serializers.ValidationError("Invalid {} address: {}".format(self.address_type, e.message))


class InetAddressField(NetfieldsField):
    netfields_type = fields.InetAddressField
    address_type = "IP"


class CidrAddressField(NetfieldsField):
    netfields_type = fields.CidrAddressField
    address_type = "CIDR"


class MACAddressField(NetfieldsField):
    netfields_type = fields.MACAddressField
    address_type = "MAC"



import django
from django.apps import AppConfig
from django.db.models import Field

from netfields.fields import CidrAddressField, InetAddressField
from netfields.lookups import (
    EndsWith,
    Family,
    IEndsWith,
    IRegex,
    IStartsWith,
    InvalidLookup,
    InvalidSearchLookup,
    MaxPrefixlen,
    MinPrefixlen,
    NetContained,
    NetContainedOrEqual,
    NetContains,
    NetContainsOrEquals,
    NetOverlaps,
    Prefixlen,
    Regex,
    StartsWith,
)


class NetfieldsConfig(AppConfig):
    name = 'netfields'

    if django.VERSION < (1, 9):
        for lookup in Field.class_lookups.keys():
            if lookup not in ['contains', 'startswith', 'endswith', 'icontains', 'istartswith', 'iendswith', 'isnull', 'in',
                              'exact', 'iexact', 'regex', 'iregex', 'lt', 'lte', 'gt', 'gte', 'equals', 'iequals', 'range', 'search']:
                invalid_lookup = InvalidLookup
                invalid_lookup.lookup_name = lookup
                CidrAddressField.register_lookup(invalid_lookup)
                InetAddressField.register_lookup(invalid_lookup)
        CidrAddressField.register_lookup(InvalidSearchLookup)
        InetAddressField.register_lookup(InvalidSearchLookup)

    CidrAddressField.register_lookup(EndsWith)
    CidrAddressField.register_lookup(IEndsWith)
    CidrAddressField.register_lookup(StartsWith)
    CidrAddressField.register_lookup(IStartsWith)
    CidrAddressField.register_lookup(Regex)
    CidrAddressField.register_lookup(IRegex)
    CidrAddressField.register_lookup(NetContained)
    CidrAddressField.register_lookup(NetContains)
    CidrAddressField.register_lookup(NetContainedOrEqual)
    CidrAddressField.register_lookup(NetContainsOrEquals)
    CidrAddressField.register_lookup(NetOverlaps)
    CidrAddressField.register_lookup(Family)
    CidrAddressField.register_lookup(MaxPrefixlen)
    CidrAddressField.register_lookup(MinPrefixlen)
    CidrAddressField.register_lookup(Prefixlen)

    InetAddressField.register_lookup(EndsWith)
    InetAddressField.register_lookup(IEndsWith)
    InetAddressField.register_lookup(StartsWith)
    InetAddressField.register_lookup(IStartsWith)
    InetAddressField.register_lookup(Regex)
    InetAddressField.register_lookup(IRegex)
    InetAddressField.register_lookup(NetContained)
    InetAddressField.register_lookup(NetContains)
    InetAddressField.register_lookup(NetContainedOrEqual)
    InetAddressField.register_lookup(NetContainsOrEquals)
    InetAddressField.register_lookup(NetOverlaps)
    InetAddressField.register_lookup(Family)



from __future__ import absolute_import, unicode_literals

from rest_framework import serializers

import unittest2 as unittest
from netfields import rest_framework as fields


class FieldsTestCase(unittest.TestCase):
    def test_validation_inet_field(self):

        class TestSerializer(serializers.Serializer):
            ip = fields.InetAddressField()

        address = '10.0.0.'
        serializer = TestSerializer(data={'ip': address})
        with self.assertRaises(serializers.ValidationError) as e:
            serializer.is_valid(raise_exception=True)
        self.assertEqual(e.exception.detail['ip'],
                         ["Invalid IP address: %r does not appear to be an IPv4 or IPv6 interface" % address])


    def test_validation_cidr_field(self):

        class TestSerializer(serializers.Serializer):
            cidr = fields.CidrAddressField()

        address = '10.0.0.'
        serializer = TestSerializer(data={'cidr': address})
        with self.assertRaises(serializers.ValidationError) as e:
            serializer.is_valid(raise_exception=True)
        self.assertEqual(e.exception.detail['cidr'],
                         ["Invalid CIDR address: %r does not appear to be an IPv4 or IPv6 network" % address])

    def test_validation_mac_field(self):

        class TestSerializer(serializers.Serializer):
            mac = fields.MACAddressField()

        address = 'de:'
        serializer = TestSerializer(data={'mac': address})
        with self.assertRaises(serializers.ValidationError) as e:
            serializer.is_valid(raise_exception=True)
        self.assertEqual(e.exception.detail['mac'], ["Invalid MAC address: failed to detect EUI version: %r" % address])

    def test_validation_additional_validators(self):
        def validate(value):
            raise serializers.ValidationError('Invalid.')

        class TestSerializer(serializers.Serializer):
            ip = fields.InetAddressField(validators=[validate])

        address = 'de:'
        serializer = TestSerializer(data={'ip': address})
        with self.assertRaises(serializers.ValidationError) as e:
            serializer.is_valid(raise_exception=True)
        self.assertItemsEqual(e.exception.detail['ip'],
                              ["Invalid IP address: %r does not appear to be an IPv4 or IPv6 interface" % address,
                               'Invalid.'])



from django.contrib.postgres.fields import ArrayField
from django.db.models import Model, ForeignKey

from netfields import InetAddressField, CidrAddressField, MACAddressField, \
        NetManager


class InetTestModel(Model):
    field = InetAddressField()
    objects = NetManager()

    class Meta:
        db_table = 'inet'


class NullInetTestModel(Model):
    field = InetAddressField(null=True)
    objects = NetManager()

    class Meta:
        db_table = 'nullinet'


class UniqueInetTestModel(Model):
    field = InetAddressField(unique=True)
    objects = NetManager()

    class Meta:
        db_table = 'uniqueinet'


class NoPrefixInetTestModel(Model):
    field = InetAddressField(store_prefix_length=False)
    objects = NetManager()

    class Meta:
        db_table = 'noprefixinet'


class CidrTestModel(Model):
    field = CidrAddressField()
    objects = NetManager()

    class Meta:
        db_table = 'cidr'


class NullCidrTestModel(Model):
    field = CidrAddressField(null=True)
    objects = NetManager()

    class Meta:
        db_table = 'nullcidr'


class UniqueCidrTestModel(Model):
    field = CidrAddressField(unique=True)
    objects = NetManager()

    class Meta:
        db_table = 'uniquecidr'


class MACTestModel(Model):
    field = MACAddressField(null=True)
    objects = NetManager()

    class Meta:
        db_table = 'mac'


class InetArrayTestModel(Model):
    field = ArrayField(InetAddressField(), blank=True, null=True)

    class Meta:
        db_table = 'inetarray'


class CidrArrayTestModel(Model):
    field = ArrayField(CidrAddressField(), blank=True, null=True)

    class Meta:
        db_table = 'cidrarray'


class MACArrayTestModel(Model):
    field = ArrayField(MACAddressField(), blank=True, null=True)

    class Meta:
        db_table = 'macarray'


class AggregateTestModel(Model):
    pass


class AggregateTestChildModel(Model):
    parent = ForeignKey('AggregateTestModel', related_name='children')
    network = CidrAddressField()
    inet = InetAddressField()
